@@ -1,8 +1,12 @@
 #!/bin/sh
 
-if [ -z "$ANDROID_NDK" -o -z "$ANDROID_ABI" ]; then
-    echo "Please set the ANDROID_NDK environment variable with its path.\n"
-    echo "ANDROID_ABI should match your ABI: armeabi-v7a, armeabi or ..."
+if [ -z "$ANDROID_NDK" ]; then
+    echo "Please set the ANDROID_NDK environment variable with its path."
+    exit 1
+fi
+
+if [ -z "$ANDROID_ABI" ]; then
+    echo "Please set ANDROID_ABI to your architecture: armeabi-v7a, armeabi, x86 or mips."
     exit 1
 fi
 
@@ -43,7 +47,9 @@ NM="${CROSS_COMPILE}nm" \
 STRIP="${CROSS_COMPILE}strip" \
 RANLIB="${CROSS_COMPILE}ranlib" \
 AR="${CROSS_COMPILE}ar" \
+PKG_CONFIG_LIBDIR=$VLC_SOURCEDIR/contrib/$TARGET_TUPLE/lib/pkgconfig \
 sh $VLC_SOURCEDIR/configure --host=$TARGET_TUPLE --build=x86_64-unknown-linux $EXTRA_PARAMS \
+                --disable-nls \
                 --enable-live555 --enable-realrtsp \
                 --enable-avformat \
                 --enable-swscale \
@@ -95,11 +101,12 @@ sh $VLC_SOURCEDIR/configure --host=$TARGET_TUPLE --build=x86_64-unknown-linux $E
                 --enable-libxml2 \
                 --disable-caca \
                 --disable-glx \
-                --disable-egl \
+                --enable-egl \
+                --enable-gles2 \
                 --disable-goom \
                 --disable-projectm \
                 --disable-sout \
-                --disable-vorbis \
+                --enable-vorbis \
                 --disable-faad \
                 --disable-x264 \
                 --disable-schroedinger --disable-dirac \
