@@ -34,6 +34,14 @@ public class LibVLC {
     public static final int AOUT_AUDIOTRACK = 1;
     public static final int AOUT_OPENSLES = 2;
 
+    public static final int VOUT_ANDROID_SURFACE = 0;
+    public static final int VOUT_OPEGLES2 = 1;
+
+    public static final int HW_ACCELERATION_AUTOMATIC = -1;
+    public static final int HW_ACCELERATION_DISABLED = 0;
+    public static final int HW_ACCELERATION_DECODING = 1;
+    public static final int HW_ACCELERATION_FULL = 2;
+
     private static LibVLC sInstance;
 
     /** libVLC instance C pointer */
@@ -41,8 +49,9 @@ public class LibVLC {
     /** libvlc_media_player pointer and index */
     private int mInternalMediaPlayerIndex = 0; // Read-only, reserved for JNI
     private long mInternalMediaPlayerInstance = 0; // Read-only, reserved for JNI
-    /** libvlc_media_list_t pointer */
-    private long mMediaListInstance = 0; // Read-only, reserved for JNI
+
+    private MediaList mMediaList; // Pointer to media list being followed
+    private MediaList mPrimaryList; // Primary/default media list; see getPrimaryMediaList()
 
     /** Buffer for VLC messages */
     private StringBuffer mDebugLogBuffer;
@@ -54,20 +63,29 @@ public class LibVLC {
     //private WakeLock mWakeLock;
 
     /** Settings */
-    private boolean iomx = false;
+    private int hardwareAcceleration = HW_ACCELERATION_AUTOMATIC;
     private String subtitlesEncoding = "";
     private int aout = LibVlcUtil.isGingerbreadOrLater() ? AOUT_OPENSLES : AOUT_AUDIOTRACK_JAVA;
+    private int vout = VOUT_ANDROID_SURFACE;
     private boolean timeStretching = false;
     private int deblocking = -1;
     private String chroma = "";
     private boolean verboseMode = true;
+    private float[] equalizer = null;
+    private boolean frameSkip = false;
+    private int networkCaching = 0;
 
     /** Check in libVLC already initialized otherwise crash */
     private boolean mIsInitialized = false;
-    public native void attachSurface(Surface surface, IVideoPlayer player, int width, int height);
+    public native void attachSurface(Surface surface, IVideoPlayer player);
 
     public native void detachSurface();
 
+    public native void attachSubtitlesSurface(Surface surface);
+    public native void detachSubtitlesSurface();
+
+    public native void eventVideoPlayerActivityCreated(boolean created);
+
     /* Load library before object instantiation */
     static {
         try {
@@ -75,7 +93,7 @@ public class LibVLC {
                 System.loadLibrary("iomx-gingerbread");
             else if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.HONEYCOMB_MR2)
                 System.loadLibrary("iomx-hc");
-            else
+            else if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.JELLY_BEAN_MR2)
                 System.loadLibrary("iomx-ics");
         } catch (Throwable t) {
             Log.w(TAG, "Unable to load the iomx library: " + t);
@@ -144,6 +162,49 @@ public class LibVLC {
         }
     }
 
+    /**
+     * Get the media list that LibVLC is following right now.
+     *
+     * @return The media list object being followed
+     */
+    public MediaList getMediaList() {
+        return mMediaList;
+    }
+
+    /**
+     * Set the media list for LibVLC to follow.
+     *
+     * @param mediaList The media list object to follow
+     */
+    public void setMediaList(MediaList mediaList) {
+        mMediaList = mediaList;
+    }
+
+    /**
+     * Sets LibVLC to follow the default media list (see below)
+     */
+    public void setMediaList() {
+        mMediaList = mPrimaryList;
+    }
+
+    /**
+     * Gets the primary media list, or the "currently playing" list.
+     * Not to be confused with the media list pointer from above, which
+     * refers the the MediaList object that libVLC is currently following.
+     * This list is just one out of many lists that it can be pointed towards.
+     *
+     * This list will be used for lists of songs that are not user-defined.
+     * For example: selecting a song from the Songs list, or from the list
+     * displayed after selecting an album.
+     *
+     * It is loaded as the default list.
+     *
+     * @return The primary media list
+     */
+    public MediaList getPrimaryMediaList() {
+        return mPrimaryList;
+    }
+
     /**
      * Give to LibVLC the surface to draw the video.
      * @param f the surface to draw
@@ -165,12 +226,20 @@ public class LibVLC {
      * those get/is* are called from native code to get settings values.
      */
 
-    public boolean useIOMX() {
-        return iomx;
+    public int getHardwareAcceleration() {
+        return this.hardwareAcceleration;
     }
 
-    public void setIomx(boolean iomx) {
-        this.iomx = iomx;
+    public void setHardwareAcceleration(int hardwareAcceleration) {
+        if (hardwareAcceleration < 0) {
+            // Automatic mode: activate MediaCodec opaque direct rendering for 4.3 and above.
+            if (LibVlcUtil.isJellyBeanMR2OrLater())
+                this.hardwareAcceleration = HW_ACCELERATION_FULL;
+            else
+                this.hardwareAcceleration = HW_ACCELERATION_DISABLED;
+        }
+        else
+            this.hardwareAcceleration = hardwareAcceleration;
     }
 
     public String getSubtitlesEncoding() {
@@ -187,11 +256,22 @@ public class LibVLC {
 
     public void setAout(int aout) {
         if (aout < 0)
-            this.aout = LibVlcUtil.isGingerbreadOrLater() ? AOUT_OPENSLES : AOUT_AUDIOTRACK_JAVA;
+            this.aout = LibVlcUtil.isICSOrLater() ? AOUT_OPENSLES : AOUT_AUDIOTRACK_JAVA;
         else
             this.aout = aout;
     }
 
+    public int getVout() {
+        return vout;
+    }
+
+    public void setVout(int vout) {
+        if (vout < 0)
+            this.vout = VOUT_ANDROID_SURFACE;
+        else
+            this.vout = vout;
+    }
+
     public boolean timeStretchingEnabled() {
         return timeStretching;
     }
@@ -243,8 +323,45 @@ public class LibVLC {
         this.verboseMode = verboseMode;
     }
 
+    public float[] getEqualizer()
+    {
+        return equalizer;
+    }
+
+    public void setEqualizer(float[] equalizer)
+    {
+        this.equalizer = equalizer;
+        applyEqualizer();
+    }
+
+    private void applyEqualizer()
+    {
+        setNativeEqualizer(mInternalMediaPlayerInstance, this.equalizer);
+    }
+    private native int setNativeEqualizer(long mediaPlayer, float[] bands);
+
+    public boolean frameSkipEnabled() {
+        return frameSkip;
+    }
+
+    public void setFrameSkip(boolean frameskip) {
+        this.frameSkip = frameskip;
+    }
+
+    public int getNetworkCaching() {
+        return this.networkCaching;
+    }
+
+    public void setNetworkCaching(int networkcaching) {
+        this.networkCaching = networkcaching;
+    }
+
     /**
-     * Initialize the libVLC class
+     * Initialize the libVLC class.
+     *
+     * This function must be called before using any libVLC functions.
+     *
+     * @throws LibVlcException
      */
     public void init(Context context) throws LibVlcException {
         Log.v(TAG, "Initializing LibVLC");
@@ -255,6 +372,7 @@ public class LibVLC {
                 throw new LibVlcException();
             }
             nativeInit();
+            mMediaList = mPrimaryList = new MediaList(this);
             setEventHandler(EventHandler.getInstance());
             mIsInitialized = true;
         }
@@ -306,29 +424,30 @@ public class LibVLC {
         mAout.release();
     }
 
-    public void readMedia(String mrl) {
-        readMedia(mLibVlcInstance, mrl, false);
-    }
-
-    /**
-     * Read a media.
-     */
-    public int readMedia(String mrl, boolean novideo) {
-        Log.v(TAG, "Reading " + mrl);
-        return readMedia(mLibVlcInstance, mrl, novideo);
-    }
-
     /**
      * Play a media from the media list (playlist)
      *
      * @param position The index of the media
      */
     public void playIndex(int position) {
-        playIndex(mLibVlcInstance, position);
+        String mrl = mMediaList.getMRL(position);
+        if (mrl == null)
+            return;
+        String[] options = mMediaList.getMediaOptions(position);
+        mInternalMediaPlayerIndex = position;
+        playMRL(mLibVlcInstance, mrl, options);
     }
 
-    public String[] readMediaMeta(String mrl) {
-        return readMediaMeta(mLibVlcInstance, mrl);
+    /**
+     * Play an MRL directly.
+     *
+     * @param mrl MRL of the media to play.
+     */
+    public void playMRL(String mrl) {
+        // index=-1 will return options from libvlc instance without relying on MediaList
+        String[] options = mMediaList.getMediaOptions(-1);
+        mInternalMediaPlayerIndex = 0;
+        playMRL(mLibVlcInstance, mrl, options);
     }
 
     public TrackInfo[] readTracksInfo(String mrl) {
@@ -349,13 +468,6 @@ public class LibVLC {
         return hasVideoTrack(mLibVlcInstance, mrl);
     }
 
-    /**
-     * Return the length of the stream, in milliseconds
-     */
-    public long getLengthFromLocation(String mrl) {
-        return getLengthFromLocation(mLibVlcInstance, mrl);
-    }
-
     /**
      * Sets the speed of playback (1 being normal speed, 2 being twice as fast)
      *
@@ -397,30 +509,10 @@ public class LibVLC {
         return mIsBufferingLog;
     }
 
-    /** Media list functions **/
-    public void removeIndex(int position) {
-        removeIndex(mMediaListInstance, position);
-    }
-    private native void removeIndex(long media_list_instance, int position);
-
     /**
-     * Read a media
-     * @param instance: the instance of libVLC
-     * @param mrl: the media mrl
-     * @param novideo: don't enable video decoding for this media
-     * @return the position in the playlist
+     * Play an mrl
      */
-    private native int readMedia(long instance, String mrl, boolean novideo);
-
-    /**
-     * Play an index in the native media list (playlist)
-     */
-    private native void playIndex(long instance, int position);
-
-    /**
-     * Return true if there is currently a running media player.
-     */
-    public native boolean hasMediaPlayer();
+    private native void playMRL(long instance, String mrl, String[] mediaOptions);
 
     /**
      * Returns true if any media is playing
@@ -447,16 +539,6 @@ public class LibVLC {
      */
     public native void stop();
 
-    /**
-     * Play the previous media (if any) in the media list
-     */
-    public native void previous();
-
-    /**
-     * Play the next media (if any) in the media list
-     */
-    public native void next();
-
     /**
      * Gets volume as integer
      */
@@ -528,11 +610,9 @@ public class LibVLC {
      */
     private native boolean hasVideoTrack(long instance, String mrl);
 
-    private native String[] readMediaMeta(long instance, String mrl);
-
     private native TrackInfo[] readTracksInfo(long instance, String mrl);
 
-    public native TrackInfo[] readTracksInfoPosition(int position);
+    public native TrackInfo[] readTracksInfoInternal();
 
     public native int getAudioTracksCount();
 
@@ -556,6 +636,13 @@ public class LibVLC {
 
     public static native String nativeToURI(String path);
 
+    /**
+     * Quickly converts path to URIs, which are mandatory in libVLC.
+     *
+     * @param path
+     *            The path to be converted.
+     * @return A URI representation of path
+     */
     public static String PathToURI(String path) {
         if(path == null) {
             throw new NullPointerException("Cannot convert null path!");
@@ -567,26 +654,33 @@ public class LibVLC {
 
     public native static boolean nativeIsPathDirectory(String path);
 
-    /**
-     * Get the list of existing items in the media list (playlist)
-     */
-    public native void getMediaListItems(ArrayList<String> arl);
-
      /**
-      * A function to flatten the playlist. This function checks the
-      * currently playing media. If there are any subitems, it will
-      * expand them and replace the current media.
+      * Expand and continue playing the current media.
       *
       * @return the index of the media was expanded, and -1 if no media was expanded
       */
-    public native int expandMedia();
+    public int expandAndPlay() {
+        int r = mMediaList.expandMedia(mInternalMediaPlayerIndex);
+        if(r == 0)
+            this.playIndex(mInternalMediaPlayerIndex);
+        return r;
+    }
 
     /**
-     * Return the length of the stream, in milliseconds
+     * Expand the current media.
+     * @return the index of the media was expanded, and -1 if no media was expanded
      */
-    private native long getLengthFromLocation(long instance, String mrl);
+    public int expand() {
+        return mMediaList.expandMedia(mInternalMediaPlayerIndex);
+    }
 
     private native void setEventHandler(EventHandler eventHandler);
 
     private native void detachEventHandler();
+
+    public native float[] getBands();
+
+    public native String[] getPresets();
+
+    public native float[] getPreset(int index);
 }
