@@ -1,8 +1,8 @@
 /*****************************************************************************
  * HistoryFragment.java
  *****************************************************************************
- * Copyright © 2012 VLC authors and VideoLAN
- * Copyright © 2012 Edward Wang
+ * Copyright © 2012-2013 VLC authors and VideoLAN
+ * Copyright © 2012-2013 Edward Wang
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,8 +23,6 @@ package org.videolan.vlc.gui;
 import org.videolan.libvlc.LibVLC;
 import org.videolan.vlc.AudioServiceController;
 import org.videolan.vlc.R;
-import org.videolan.vlc.gui.audio.AudioPlayerActivity;
-import android.content.Intent;
 import android.os.Bundle;
 import android.util.Log;
 import android.view.ContextMenu;
@@ -51,13 +49,15 @@ public class HistoryFragment extends SherlockListFragment {
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
 
-        mHistoryAdapter = new HistoryAdapter();
+        mHistoryAdapter = new HistoryAdapter(getActivity());
         Log.d(TAG, "HistoryFragment()");
     }
 
     @Override
     public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
     {
+        getSherlockActivity().getSupportActionBar().setTitle(R.string.history);
+
         View v = inflater.inflate(R.layout.history_list, container, false);
         setListAdapter(mHistoryAdapter);
         final ListView listView = (ListView)v.findViewById(android.R.id.list);
@@ -79,10 +79,8 @@ public class HistoryFragment extends SherlockListFragment {
     private void playListIndex(int position) {
         AudioServiceController audioController = AudioServiceController.getInstance();
 
-        audioController.load(mHistoryAdapter.getAllURIs(), position, true, true);
-        Intent intent = new Intent(getActivity(), AudioPlayerActivity.class);
-        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
-        startActivity(intent);
+        LibVLC.getExistingInstance().setMediaList();
+        audioController.playIndex(position);
     }
 
     @Override
@@ -98,8 +96,8 @@ public class HistoryFragment extends SherlockListFragment {
             playListIndex(info.position);
             return true;
         } else if(id == R.id.history_view_delete) {
-            LibVLC.getExistingInstance().removeIndex(info.position);
-            mHistoryAdapter.notifyDataSetChanged();
+            LibVLC.getExistingInstance().getPrimaryMediaList().remove(info.position);
+            mHistoryAdapter.refresh();
             return true;
         }
         return super.onContextItemSelected(item);
