@@ -429,21 +429,23 @@ public class MainActivity extends SherlockFragmentActivity {
         if (slideDownAudioPlayer())
             return;
 
-        // If it's the directory view, a "backpressed" action shows a parent.
-        if (mCurrentFragment.equals("directories")) {
-            DirectoryViewFragment directoryView = (DirectoryViewFragment) getFragment(mCurrentFragment);
-            if (!directoryView.isRootDirectory()) {
-                directoryView.showParentDirectory();
+        if (mCurrentFragment!= null) {
+            // If it's the directory view, a "backpressed" action shows a parent.
+            if (mCurrentFragment.equals("directories")) {
+                DirectoryViewFragment directoryView = (DirectoryViewFragment) getFragment(mCurrentFragment);
+                if (!directoryView.isRootDirectory()) {
+                    directoryView.showParentDirectory();
+                    return;
+                }
+            }
+
+            // If it's the albums songs fragment, we leave it.
+            if (secondaryFragments.contains(mCurrentFragment)) {
+                popSecondaryFragment();
                 return;
             }
         }
 
-        // If it's the albums songs fragment, we leave it.
-        if (secondaryFragments.contains(mCurrentFragment)) {
-            popSecondaryFragment();
-            return;
-        }
-
         super.onBackPressed();
     }
 
@@ -516,12 +518,14 @@ public class MainActivity extends SherlockFragmentActivity {
         // Slide down the audio player if needed.
         slideDownAudioPlayer();
 
-        // Do not show the new fragment if the requested fragment is already shown.
-        if (mCurrentFragment.equals(fragmentTag))
-            return null;
+        if (mCurrentFragment != null) {
+            // Do not show the new fragment if the requested fragment is already shown.
+            if (mCurrentFragment.equals(fragmentTag))
+                return null;
 
-        if (!secondaryFragments.contains(mCurrentFragment))
-            mPreviousFragment = mCurrentFragment;
+            if (!secondaryFragments.contains(mCurrentFragment))
+                mPreviousFragment = mCurrentFragment;
+        }
 
         mCurrentFragment = fragmentTag;
         Fragment frag = fetchSecondaryFragment(mCurrentFragment);
@@ -577,7 +581,7 @@ public class MainActivity extends SherlockFragmentActivity {
 
     @Override
     public boolean onSearchRequested() {
-        if (mCurrentFragment.equals("search"))
+        if (mCurrentFragment != null && mCurrentFragment.equals("search"))
             ((SearchFragment)fetchSecondaryFragment("search")).onSearchKeyPressed();
         showSecondaryFragment("search");
         return true;
