@@ -1759,7 +1759,7 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
                     // other content-based URI (probably file pickers)
                     mLocation = getIntent().getData().getPath();
                 }
-            } else {
+            } else if (getIntent().getDataString() != null) {
                 // Plain URI
                 mLocation = getIntent().getDataString();
                 // Remove VLC prefix if needed
@@ -1774,6 +1774,9 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
                         e.printStackTrace();
                     }
                 }
+            } else {
+                Log.e(TAG, "Couldn't understand the intent");
+                encounteredError();
             }
             if(getIntent().getExtras() != null)
                 intentPosition = getIntent().getExtras().getLong("position", -1);
@@ -1985,7 +1988,7 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
                 + "has a presentation display.");
         mLibVLC.pause(); // Stop sending frames to avoid a crash.
         finish(); //TODO restore the video on the new display instead of closing
-        mPresentation.dismiss();
+        if (mPresentation != null) mPresentation.dismiss();
         mPresentation = null;
     }
 
