@@ -27,12 +27,24 @@ import android.graphics.Bitmap;
 public interface IAudioPlayerControl {
     String getTitle();
 
+    String getTitlePrev();
+
+    String getTitleNext();
+
     String getArtist();
 
+    String getArtistPrev();
+
+    String getArtistNext();
+
     String getAlbum();
 
     Bitmap getCover();
 
+    Bitmap getCoverPrev();
+
+    Bitmap getCoverNext();
+
     int getLength();
 
     int getTime();
